@@ -1,7 +1,7 @@
 // qsamplerAbout.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2005, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -13,9 +13,9 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *****************************************************************************/
 
@@ -29,8 +29,12 @@
 
 #define QSAMPLER_SUBTITLE     "A LinuxSampler Qt GUI Interface"
 #define QSAMPLER_WEBSITE      "http://qsampler.sourceforge.net"
-#define QSAMPLER_COPYRIGHT    "Copyright (C) 2004-2006, rncbc aka Rui Nuno Capela. All rights reserved."
+#define QSAMPLER_COPYRIGHT    "Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved."
+#define QSAMPLER_COPYRIGHT2   "Copyright (C) 2007, Christian Schoenebeck"
+
+#define QSAMPLER_DOMAIN       "linuxsampler.org"
 
 #endif  // __qsamplerAbout_h
 
+
 // end of qsamplerAbout.h
