@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -926,7 +926,7 @@ ChannelRoutingModel::ChannelRoutingModel ( QObject *pParent )
 
 int ChannelRoutingModel::rowCount ( const QModelIndex& /*parent*/) const
 {
-	return m_routing.size();
+	return (m_pDevice) ? m_routing.size() : 0;
 }
 
 
@@ -994,7 +994,7 @@ QVariant ChannelRoutingModel::headerData ( int section,
 		case Qt::Horizontal:
 			return UNICODE_RIGHT_ARROW + QObject::tr(" Device Channel");
 		case Qt::Vertical:
-			return QObject::tr("Sampler Channel ") +
+			return QObject::tr("Audio Channel ") +
 				QString::number(section) + " " + UNICODE_RIGHT_ARROW;
 		default:
 			return QVariant();
