@@ -1,7 +1,8 @@
 // qsamplerChannel.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -13,51 +14,43 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 *****************************************************************************/
 
 #ifndef __qsamplerChannel_h
 #define __qsamplerChannel_h
 
-#include <qtable.h>
+#include <QTableWidgetItem>
+#include <QItemDelegate>
 
 #include <lscp/client.h>
 #include <lscp/device.h>
 
 #include "qsamplerOptions.h"
 
-class qsamplerDevice;
-class qsamplerMainForm;
+namespace QSampler {
 
+class Device;
 
 // Typedef'd QMap.
-typedef QMap<int, int> qsamplerChannelRoutingMap;
+typedef QMap<int, int> ChannelRoutingMap;
 
 
 //-------------------------------------------------------------------------
-// qsamplerChannel - Sampler channel structure.
+// QSampler::Channel - Sampler channel structure.
 //
 
-class qsamplerChannel
+class Channel
 {
 public:
 
 	// Constructor.
-	qsamplerChannel(qsamplerMainForm *pMainForm, int iChannelID = -1);
+	Channel(int iChannelID = -1);
 	// Default destructor.
-	~qsamplerChannel();
-
-	// Main application form accessor.
-	qsamplerMainForm *mainForm() const;
-
-	// Main application options accessor.
-	qsamplerOptions *options() const;
-
-	// LSCP client descriptor accessor.
-	lscp_client_t * client() const;
+	~Channel();
 
 	// Add/remove sampler channel methods.
 	bool     addChannel();
@@ -101,6 +94,10 @@ public:
 	int      midiChannel() const;
 	bool     setMidiChannel(int iMidiChannel);
 
+	// MIDI instrument map.
+	int      midiMap() const;
+	bool     setMidiMap(int iMidiMap);
+
 	// Audio output driver (DEPRECATED).
 	const QString& audioDriver() const;
 	bool     setAudioDriver(const QString& sAudioDriver);
@@ -125,7 +122,7 @@ public:
 	int      audioChannel(int iAudioOut) const;
 	bool     setAudioChannel(int iAudioOut, int iAudioIn);
 	// The audio routing map itself.
-	const qsamplerChannelRoutingMap& audioRouting() const;
+	const ChannelRoutingMap& audioRouting() const;
 
 	// Istrument name remapper.
 	void     updateInstrumentName();
@@ -139,6 +136,9 @@ public:
 	// Reset channel method.
 	bool     channelReset();
 
+	// Spawn instrument editor method.
+	bool     editChannel();
+
 	// Message logging methods (brainlessly mapped to main form's).
 	void     appendMessages       (const QString & s) const;
 	void     appendMessagesColor  (const QString & s, const QString & c) const;
@@ -165,9 +165,6 @@ public:
 
 private:
 
-	// Main application form reference.
-	qsamplerMainForm *m_pMainForm;
-
 	// Unique channel identifier.
 	int     m_iChannelID;
 
@@ -181,6 +178,7 @@ private:
 	int     m_iMidiDevice;
 	int     m_iMidiPort;
 	int     m_iMidiChannel;
+	int     m_iMidiMap;
 	QString m_sAudioDriver;
 	int     m_iAudioDevice;
 	float   m_fVolume;
@@ -188,63 +186,79 @@ private:
 	bool    m_bSolo;
 
 	// The audio routing mapping.
-	qsamplerChannelRoutingMap m_audioRouting;
+	ChannelRoutingMap m_audioRouting;
 };
 
 
 //-------------------------------------------------------------------------
-// qsamplerChannelRoutingTable - Channel routing table widget.
+// QSampler::ChannelRoutingModel - data model for audio routing
+//                                 (used for QTableView)
 //
 
-class qsamplerChannelRoutingTable : public QTable
+struct ChannelRoutingItem {
+	QStringList options;
+	int         selection;
+};
+
+class ChannelRoutingModel : public QAbstractTableModel
+{
+	Q_OBJECT
+public:
+
+	ChannelRoutingModel(QObject* pParent = NULL);
+
+	// overridden methods from subclass(es)
+	int rowCount(const QModelIndex& parent = QModelIndex()) const;
+	int columnCount(const QModelIndex& parent = QModelIndex()) const;
+	Qt::ItemFlags flags(const QModelIndex& index) const;
+	bool setData(const QModelIndex& index, const QVariant& value,
+		int role = Qt::EditRole);
+	QVariant data(const QModelIndex &index, int role) const;
+	QVariant headerData(int section, Qt::Orientation orientation,
+		int role = Qt::DisplayRole) const;
+
+	// own methods
+	ChannelRoutingMap routingMap() const { return m_routing; }
+
+	void clear() { m_routing.clear(); }
+
+public slots:
+
+	void refresh(Device *pDevice,
+		const ChannelRoutingMap& routing);
+
+private:
+
+	Device *m_pDevice;
+	ChannelRoutingMap m_routing;
+};
+
+
+//-------------------------------------------------------------------------
+// QSampler::ChannelRoutingDelegate - table cell renderer for audio routing
+//
+
+class ChannelRoutingDelegate : public QItemDelegate
 {
 	Q_OBJECT
 
 public:
 
-	// Constructor.
-	qsamplerChannelRoutingTable(QWidget *pParent = 0, const char *pszName = 0);
-	// Default destructor.
-	~qsamplerChannelRoutingTable();
+	ChannelRoutingDelegate(QObject* pParent = NULL);
 
-	// Common parameter table renderer.
-	void refresh(qsamplerDevice *pDevice,
-		const qsamplerChannelRoutingMap& routing);
-
-	// Commit any pending editing.
-	void flush();
+	QWidget* createEditor(QWidget *pParent,
+		const QStyleOptionViewItem& option, const QModelIndex& index) const;
+	void setEditorData(QWidget *pEditor, const QModelIndex& index) const;
+	void setModelData(QWidget *pEditor, QAbstractItemModel* model,
+		const QModelIndex& index) const;
+	void updateEditorGeometry(QWidget *pEditor,
+		const QStyleOptionViewItem& option, const QModelIndex& index) const;
 };
 
+} // namespace QSampler
 
-//-------------------------------------------------------------------------
-// qsamplerChannelRoutingComboBox - Custom combo box for routing table.
-//
-
-class qsamplerChannelRoutingComboBox : public QTableItem
-{
-public:
-
-	// Constructor.
-	qsamplerChannelRoutingComboBox(QTable *pTable,
-		const QStringList& list, const QPixmap& pixmap);
-
-	// Public accessors.
-	void setCurrentItem(int iCurrentItem);
-	int currentItem() const;
-
-protected:
-
-	// Virtual implemetations.
-	QWidget *createEditor() const;
-	void setContentFromEditor(QWidget *pWidget);
-
-private:
-
-	// Initial value holders
-	QStringList m_list;
-	int m_iCurrentItem;
-};
-
+// So we can use it i.e. through QVariant
+Q_DECLARE_METATYPE(QSampler::ChannelRoutingItem)
 
 #endif  // __qsamplerChannel_h
 
