@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,9 +25,15 @@
 
 #include "qsamplerMainForm.h"
 
+#include "qsamplerChannelFxForm.h"
+
+#include <QMessageBox>
 #include <QDragEnterEvent>
 #include <QUrl>
 
+#define MIDI_OFF_COLOR			Qt::darkGreen
+#define MIDI_ON_COLOR			Qt::green
+
 // Channel status/usage usage limit control.
 #define QSAMPLER_ERROR_LIMIT	3
 
@@ -85,6 +91,24 @@ ChannelStrip::ChannelStrip ( QWidget* pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.ChannelEditPushButton,
 		SIGNAL(clicked()),
 		SLOT(channelEdit()));
+	QObject::connect(m_ui.FxPushButton,
+		SIGNAL(clicked()),
+		SLOT(channelFxEdit()));
+
+	pMidiActivityTimer = new QTimer(this);
+	pMidiActivityTimer->setSingleShot(true);
+	QObject::connect(
+		pMidiActivityTimer, SIGNAL(timeout()),
+		this, SLOT(midiDataCeased())
+	);
+
+#if CONFIG_EVENT_CHANNEL_MIDI
+	m_ui.MidiActivityLabel->setPalette(MIDI_OFF_COLOR);
+	m_ui.MidiActivityLabel->setAutoFillBackground(true);
+#else
+	m_ui.MidiActivityLabel->setText("X");
+	m_ui.MidiActivityLabel->setToolTip("MIDI Activity Disabled");
+#endif
 
 	setSelected(false);
 }
@@ -213,6 +237,7 @@ void ChannelStrip::setDisplayEffect ( bool bDisplayEffect )
 		pal.setColor(QPalette::Background, Qt::black);
 	}
 	m_ui.ChannelInfoFrame->setPalette(pal);
+	m_ui.InstrumentNameTextLabel->setPalette(pal);
 	m_ui.StreamVoiceCountTextLabel->setPalette(pal);
 }
 
@@ -284,6 +309,33 @@ void ChannelStrip::channelEdit (void)
 	m_pChannel->editChannel();
 }
 
+bool ChannelStrip::channelFxEdit (void)
+{
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !channel())
+		return false;
+
+	pMainForm->appendMessages(QObject::tr("channel fx sends..."));
+
+	bool bResult = false;
+
+#if CONFIG_FXSEND
+	ChannelFxForm *pChannelFxForm =
+		new ChannelFxForm(channel(), parentWidget());
+	if (pChannelFxForm) {
+		//pChannelForm->setup(this);
+		bResult = pChannelFxForm->exec();
+		delete pChannelFxForm;
+	}
+#else // CONFIG_FXSEND
+	QMessageBox::critical(this,
+		QSAMPLER_TITLE ": " + tr("Unavailable"),
+			tr("Sorry, QSampler was built without FX send support!\n\n"
+			   "(Make sure you have a recent liblscp when recompiling QSampler)"));
+#endif // CONFIG_FXSEND
+
+	return bResult;
+}
 
 // Channel reset slot.
 bool ChannelStrip::channelReset (void)
@@ -497,6 +549,10 @@ void ChannelStrip::volumeChanged ( int iVolume )
 	}
 }
 
+void ChannelStrip::midiArrived() {
+	m_ui.MidiActivityLabel->setPalette(MIDI_ON_COLOR);
+	pMidiActivityTimer->start(50);
+}
 
 // Context menu event handler.
 void ChannelStrip::contextMenuEvent( QContextMenuEvent *pEvent )
@@ -508,6 +564,9 @@ void ChannelStrip::contextMenuEvent( QContextMenuEvent *pEvent )
 	m_pChannel->contextMenuEvent(pEvent);
 }
 
+void ChannelStrip::midiDataCeased() {
+	m_ui.MidiActivityLabel->setPalette(MIDI_OFF_COLOR);
+}
 
 // Error count hackish accessors.
 void ChannelStrip::resetErrorCount (void)
