@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 #include "ui_qsamplerChannelStrip.h"
 
 #include "qsamplerChannel.h"
+#include <QTimer>
 
 class QDragEnterEvent;
 
@@ -77,8 +78,10 @@ public slots:
 	bool channelMute(bool bMute);
 	bool channelSolo(bool bSolo);
 	void channelEdit();
+	bool channelFxEdit();
 	bool channelReset();
 	void volumeChanged(int iVolume);
+	void midiArrived();
 
 protected:
 
@@ -86,6 +89,10 @@ protected:
 	void dropEvent(QDropEvent* pDropEvent);
 	void contextMenuEvent(QContextMenuEvent* pEvent);
 
+protected slots:
+
+	void midiDataCeased();
+
 private:
 
 	Ui::qsamplerChannelStrip m_ui;
@@ -93,6 +100,7 @@ private:
 	Channel* m_pChannel;
 	int m_iDirtyChange;
 	int m_iErrorCount;
+	QTimer* pMidiActivityTimer;
 
 	// Channel strip activation/selection.
 	static ChannelStrip *g_pSelectedStrip;
