@@ -1,8 +1,8 @@
 // qsamplerDeviceForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -279,7 +279,8 @@ void DeviceForm::deleteDevice (void)
 			"%1\n\n"
 			"Are you sure?")
 			.arg(device.deviceName()),
-			tr("OK"), tr("Cancel")) > 0)
+			QMessageBox::Ok | QMessageBox::Cancel)
+			== QMessageBox::Cancel)
 			return;
 	}
 
@@ -613,16 +614,16 @@ void DeviceForm::deviceListViewContextMenu ( const QPoint& pos )
 	bool bClient = (pMainForm->client() != NULL);
 	bool bEnabled = (pItem != NULL);
 	pAction = menu.addAction(
-		QIcon(":/qsampler/pixmaps/deviceCreate.png"),
+		QIcon(":/icons/deviceCreate.png"),
 		tr("&Create device"), this, SLOT(createDevice()));
 	pAction->setEnabled(bEnabled || (bClient && m_bNewDevice));
 	pAction = menu.addAction(
-		QIcon(":/qsampler/pixmaps/deviceDelete.png"),
+		QIcon(":/icons/deviceDelete.png"),
 		tr("&Delete device"), this, SLOT(deleteDevice()));
 	pAction->setEnabled(bEnabled && !m_bNewDevice);
 	menu.addSeparator();
 	pAction = menu.addAction(
-		QIcon(":/qsampler/pixmaps/formRefresh.png"),
+		QIcon(":/icons/formRefresh.png"),
 		tr("&Refresh"), this, SLOT(refreshDevices()));
 	pAction->setEnabled(bClient);
 
