@@ -1,8 +1,8 @@
 // qsamplerMainForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -33,6 +33,7 @@
 #include "qsamplerInstrumentListForm.h"
 #include "qsamplerDeviceForm.h"
 #include "qsamplerOptionsForm.h"
+#include "qsamplerDeviceStatusForm.h"
 
 #include <QApplication>
 #include <QWorkspace>
@@ -390,6 +391,10 @@ void MainForm::setup ( Options *pOptions )
 #else
 	viewInstrumentsAction->setEnabled(false);
 #endif
+	// Setup messages logging appropriately...
+	m_pMessages->setLogging(
+		m_pOptions->bMessagesLog,
+		m_pOptions->sMessagesLogPath);
 	// Set message defaults...
 	updateMessagesFont();
 	updateMessagesLimit();
@@ -471,7 +476,7 @@ bool MainForm::queryClose (void)
 			if (m_pDeviceForm)
 				m_pDeviceForm->close();
 			// Stop client and/or server, gracefully.
-			stopServer();
+			stopServer(true /*interactive*/);
 		}
 	}
 
@@ -481,9 +486,10 @@ bool MainForm::queryClose (void)
 
 void MainForm::closeEvent ( QCloseEvent *pCloseEvent )
 {
-	if (queryClose())
+	if (queryClose()) {
+		DeviceStatusForm::deleteAllInstances();
 		pCloseEvent->accept();
-	else
+	} else
 		pCloseEvent->ignore();
 }
 
@@ -551,19 +557,77 @@ void MainForm::customEvent(QEvent* pCustomEvent)
 	// For the time being, just pump it to messages.
 	if (pCustomEvent->type() == QSAMPLER_CUSTOM_EVENT) {
 		CustomEvent *pEvent = static_cast<CustomEvent *> (pCustomEvent);
-		if (pEvent->event() == LSCP_EVENT_CHANNEL_INFO) {
-			int iChannelID = pEvent->data().toInt();
-			ChannelStrip *pChannelStrip = channelStrip(iChannelID);
-			if (pChannelStrip)
-				channelStripChanged(pChannelStrip);
-		} else {
-			appendMessagesColor(tr("Notify event: %1 data: %2")
-				.arg(::lscp_event_to_text(pEvent->event()))
-				.arg(pEvent->data()), "#996699");
+		switch (pEvent->event()) {
+			case LSCP_EVENT_CHANNEL_COUNT:
+				updateAllChannelStrips(true);
+				break;
+			case LSCP_EVENT_CHANNEL_INFO: {
+				int iChannelID = pEvent->data().toInt();
+				ChannelStrip *pChannelStrip = channelStrip(iChannelID);
+				if (pChannelStrip)
+					channelStripChanged(pChannelStrip);
+				break;
+			}
+			case LSCP_EVENT_MIDI_INPUT_DEVICE_COUNT:
+				if (m_pDeviceForm) m_pDeviceForm->refreshDevices();
+				DeviceStatusForm::onDevicesChanged();
+				updateViewMidiDeviceStatusMenu();
+				break;
+			case LSCP_EVENT_MIDI_INPUT_DEVICE_INFO: {
+				if (m_pDeviceForm) m_pDeviceForm->refreshDevices();
+				const int iDeviceID = pEvent->data().section(' ', 0, 0).toInt();
+				DeviceStatusForm::onDeviceChanged(iDeviceID);
+				break;
+			}
+			case LSCP_EVENT_AUDIO_OUTPUT_DEVICE_COUNT:
+				if (m_pDeviceForm) m_pDeviceForm->refreshDevices();
+				break;
+			case LSCP_EVENT_AUDIO_OUTPUT_DEVICE_INFO:
+				if (m_pDeviceForm) m_pDeviceForm->refreshDevices();
+				break;
+#if CONFIG_EVENT_CHANNEL_MIDI
+			case LSCP_EVENT_CHANNEL_MIDI: {
+				const int iChannelID = pEvent->data().section(' ', 0, 0).toInt();
+				ChannelStrip *pChannelStrip = channelStrip(iChannelID);
+				if (pChannelStrip)
+					pChannelStrip->midiArrived();
+				break;
+			}
+#endif
+#if CONFIG_EVENT_DEVICE_MIDI
+			case LSCP_EVENT_DEVICE_MIDI: {
+				const int iDeviceID = pEvent->data().section(' ', 0, 0).toInt();
+				const int iPortID   = pEvent->data().section(' ', 1, 1).toInt();
+				DeviceStatusForm* pDeviceStatusForm =
+					DeviceStatusForm::getInstance(iDeviceID);
+				if (pDeviceStatusForm)
+					pDeviceStatusForm->midiArrived(iPortID);
+				break;
+			}
+#endif
+			default:
+				appendMessagesColor(tr("Notify event: %1 data: %2")
+					.arg(::lscp_event_to_text(pEvent->event()))
+					.arg(pEvent->data()), "#996699");
 		}
 	}
 }
 
+void MainForm::updateViewMidiDeviceStatusMenu() {
+	m_ui.viewMidiDeviceStatusMenu->clear();
+	const std::map<int, DeviceStatusForm*> statusForms =
+		DeviceStatusForm::getInstances();
+	for (
+		std::map<int, DeviceStatusForm*>::const_iterator iter = statusForms.begin();
+		iter != statusForms.end(); ++iter
+	) {
+		DeviceStatusForm* pForm = iter->second;
+		m_ui.viewMidiDeviceStatusMenu->addAction(
+			pForm->visibleAction()
+		);
+	}
+}
+
 // Context menu event handler.
 void MainForm::contextMenuEvent( QContextMenuEvent *pEvent )
 {
@@ -695,7 +759,8 @@ bool MainForm::saveSession ( bool bPrompt )
 				"\"%1\"\n\n"
 				"Do you want to replace it?")
 				.arg(sFilename),
-				tr("Replace"), tr("Cancel")) > 0)
+				QMessageBox::Yes | QMessageBox::No)
+				== QMessageBox::No)
 				return false;
 		}
 	}
@@ -718,11 +783,13 @@ bool MainForm::closeSession ( bool bForce )
 			"\"%1\"\n\n"
 			"Do you want to save the changes?")
 			.arg(sessionName(m_sFilename)),
-			tr("Save"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Save...
+			QMessageBox::Save |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Save:
 			bClose = saveSession(false);
 			// Fall thru....
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bClose = false;
@@ -1243,7 +1310,8 @@ void MainForm::fileReset (void)
 		"Please note that this operation may cause\n"
 		"temporary MIDI and Audio disruption.\n\n"
 		"Do you want to reset the sampler engine now?"),
-		tr("Reset"), tr("Cancel")) > 0)
+		QMessageBox::Ok | QMessageBox::Cancel)
+		== QMessageBox::Cancel)
 		return;
 
 	// Trye closing the current session, first...
@@ -1284,7 +1352,7 @@ void MainForm::fileRestart (void)
 			"Please note that this operation may cause\n"
 			"temporary MIDI and Audio disruption.\n\n"
 			"Do you want to restart the connection now?"),
-			tr("Restart"), tr("Cancel")) == 0);
+			QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok);
 	}
 
 	// Are we still for it?
@@ -1365,7 +1433,8 @@ void MainForm::editRemoveChannel (void)
 			"%1\n\n"
 			"Are you sure?")
 			.arg(pChannelStrip->windowTitle()),
-			tr("OK"), tr("Cancel")) > 0)
+			QMessageBox::Ok | QMessageBox::Cancel)
+			== QMessageBox::Cancel)
 			return;
 	}
 
@@ -1556,6 +1625,8 @@ void MainForm::viewOptions (void)
 		int     iOldServerTimeout   = m_pOptions->iServerTimeout;
 		bool    bOldServerStart     = m_pOptions->bServerStart;
 		QString sOldServerCmdLine   = m_pOptions->sServerCmdLine;
+		bool    bOldMessagesLog     = m_pOptions->bMessagesLog;
+		QString sOldMessagesLogPath = m_pOptions->sMessagesLogPath;
 		QString sOldDisplayFont     = m_pOptions->sDisplayFont;
 		bool    bOldDisplayEffect   = m_pOptions->bDisplayEffect;
 		int     iOldMaxVolume       = m_pOptions->iMaxVolume;
@@ -1567,6 +1638,7 @@ void MainForm::viewOptions (void)
 		bool    bOldCompletePath    = m_pOptions->bCompletePath;
 		bool    bOldInstrumentNames = m_pOptions->bInstrumentNames;
 		int     iOldMaxRecentFiles  = m_pOptions->iMaxRecentFiles;
+		int     iOldBaseFontSize    = m_pOptions->iBaseFontSize;
 		// Load the current setup settings.
 		pOptionsForm->setup(m_pOptions);
 		// Show the setup dialog...
@@ -1575,14 +1647,20 @@ void MainForm::viewOptions (void)
 			if (( bOldStdoutCapture && !m_pOptions->bStdoutCapture) ||
 				(!bOldStdoutCapture &&  m_pOptions->bStdoutCapture) ||
 				( bOldKeepOnTop     && !m_pOptions->bKeepOnTop)     ||
-				(!bOldKeepOnTop     &&  m_pOptions->bKeepOnTop)) {
+				(!bOldKeepOnTop     &&  m_pOptions->bKeepOnTop)     ||
+				(iOldBaseFontSize   !=  m_pOptions->iBaseFontSize)) {
 				QMessageBox::information(this,
 					QSAMPLER_TITLE ": " + tr("Information"),
 					tr("Some settings may be only effective\n"
-					"next time you start this program."), tr("OK"));
+					"next time you start this program."));
 				updateMessagesCapture();
 			}
 			// Check wheather something immediate has changed.
+			if (( bOldMessagesLog && !m_pOptions->bMessagesLog) ||
+				(!bOldMessagesLog &&  m_pOptions->bMessagesLog) ||
+				(sOldMessagesLogPath != m_pOptions->sMessagesLogPath))
+				m_pMessages->setLogging(
+					m_pOptions->bMessagesLog, m_pOptions->sMessagesLogPath);
 			if (( bOldCompletePath && !m_pOptions->bCompletePath) ||
 				(!bOldCompletePath &&  m_pOptions->bCompletePath) ||
 				(iOldMaxRecentFiles != m_pOptions->iMaxRecentFiles))
@@ -1736,6 +1814,21 @@ void MainForm::helpAbout (void)
 	sText += "<small><font color=\"red\">";
 	sText += tr("Instrument editing support disabled.");
 	sText += "</font></small><br />";
+#endif
+#ifndef CONFIG_EVENT_CHANNEL_MIDI
+	sText += "<small><font color=\"red\">";
+	sText += tr("Channel MIDI event support disabled.");
+	sText += "</font></small><br />";
+#endif
+#ifndef CONFIG_EVENT_DEVICE_MIDI
+	sText += "<small><font color=\"red\">";
+	sText += tr("Device MIDI event support disabled.");
+	sText += "</font></small><br />";
+#endif
+#ifndef CONFIG_MAX_VOICES
+	sText += "<small><font color=\"red\">";
+	sText += tr("Runtime max. voices / disk streams support disabled.");
+	sText += "</font></small><br />";
 #endif
 	sText += "<br />\n";
 	sText += tr("Using") + ": ";
@@ -1911,6 +2004,20 @@ void MainForm::updateSession (void)
 	}
 #endif
 
+	updateAllChannelStrips(false);
+
+	// Do we auto-arrange?
+	if (m_pOptions && m_pOptions->bAutoArrange)
+		channelsArrange();
+
+	// Remember to refresh devices and instruments...
+	if (m_pInstrumentListForm)
+		m_pInstrumentListForm->refreshInstruments();
+	if (m_pDeviceForm)
+		m_pDeviceForm->refreshDevices();
+}
+
+void MainForm::updateAllChannelStrips(bool bRemoveDeadStrips) {
 	// Retrieve the current channel list.
 	int *piChannelIDs = ::lscp_list_channels(m_pClient);
 	if (piChannelIDs == NULL) {
@@ -1927,21 +2034,33 @@ void MainForm::updateSession (void)
 			if (!channelStrip(piChannelIDs[iChannel]))
 				createChannelStrip(new Channel(piChannelIDs[iChannel]));
 		}
+
+		// Do we auto-arrange?
+		if (m_pOptions && m_pOptions->bAutoArrange)
+			channelsArrange();
+
+		stabilizeForm();
+
+		// remove dead channel strips
+		if (bRemoveDeadStrips) {
+			for (int i = 0; channelStripAt(i); ++i) {
+				ChannelStrip* pChannelStrip = channelStripAt(i);
+				bool bExists = false;
+				for (int j = 0; piChannelIDs[j] >= 0; ++j) {
+					if (!pChannelStrip->channel()) break;
+					if (piChannelIDs[j] == pChannelStrip->channel()->channelID()) {
+						// strip exists, don't touch it
+						bExists = true;
+						break;
+					}
+				}
+				if (!bExists) destroyChannelStrip(pChannelStrip);
+			}
+		}
 		m_pWorkspace->setUpdatesEnabled(true);
 	}
-
-	// Do we auto-arrange?
-	if (m_pOptions && m_pOptions->bAutoArrange)
-		channelsArrange();
-
-	// Remember to refresh devices and instruments...
-	if (m_pInstrumentListForm)
-		m_pInstrumentListForm->refreshInstruments();
-	if (m_pDeviceForm)
-		m_pDeviceForm->refreshDevices();
 }
 
-
 // Update the recent files list and menu.
 void MainForm::updateRecentFiles ( const QString& sFilename )
 {
@@ -2114,7 +2233,7 @@ void MainForm::appendMessagesError( const QString& s )
 	QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 
 	QMessageBox::critical(this,
-		QSAMPLER_TITLE ": " + tr("Error"), s, tr("Cancel"));
+		QSAMPLER_TITLE ": " + tr("Error"), s, QMessageBox::Cancel);
 }
 
 
@@ -2219,6 +2338,16 @@ ChannelStrip* MainForm::createChannelStrip ( Channel *pChannel )
 	return pChannelStrip;
 }
 
+void MainForm::destroyChannelStrip(ChannelStrip* pChannelStrip) {
+	// Just delete the channel strip.
+	delete pChannelStrip;
+
+	// Do we auto-arrange?
+	if (m_pOptions && m_pOptions->bAutoArrange)
+		channelsArrange();
+
+	stabilizeForm();
+}
 
 // Retrieve the active channel strip.
 ChannelStrip* MainForm::activeChannelStrip (void)
@@ -2230,11 +2359,16 @@ ChannelStrip* MainForm::activeChannelStrip (void)
 // Retrieve a channel strip by index.
 ChannelStrip* MainForm::channelStripAt ( int iChannel )
 {
+	if (!m_pWorkspace) return NULL;
+
 	QWidgetList wlist = m_pWorkspace->windowList();
 	if (wlist.isEmpty())
 		return NULL;
 
-	return static_cast<ChannelStrip *> (wlist.at(iChannel));
+	if (iChannel < 0 || iChannel >= wlist.size())
+		return NULL;
+
+	return dynamic_cast<ChannelStrip *> (wlist.at(iChannel));
 }
 
 
@@ -2387,17 +2521,13 @@ void MainForm::startServer (void)
 
 	// Is the server process instance still here?
 	if (m_pServer) {
-		switch (QMessageBox::warning(this,
+		if (QMessageBox::warning(this,
 			QSAMPLER_TITLE ": " + tr("Warning"),
 			tr("Could not start the LinuxSampler server.\n\n"
-			"Maybe it ss already started."),
-			tr("Stop"), tr("Kill"), tr("Cancel"))) {
-		case 0:
+			"Maybe it is already started."),
+			QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok) {
 			m_pServer->terminate();
-			break;
-		case 1:
 			m_pServer->kill();
-			break;
 		}
 		return;
 	}
@@ -2410,7 +2540,8 @@ void MainForm::startServer (void)
 		return;
 
 	// OK. Let's build the startup process...
-	m_pServer = new QProcess(this);
+	m_pServer = new QProcess();
+	bForceServerStop = true;
 
 	// Setup stdout/stderr capture...
 //	if (m_pOptions->bStdoutCapture) {
@@ -2458,13 +2589,27 @@ void MainForm::startServer (void)
 
 
 // Stop linuxsampler server...
-void MainForm::stopServer (void)
+void MainForm::stopServer (bool bInteractive)
 {
 	// Stop client code.
 	stopClient();
 
+	if (m_pServer && bInteractive) {
+		if (QMessageBox::question(this,
+			QSAMPLER_TITLE ": " + tr("The backend's fate ..."),
+			tr("You have the option to keep the sampler backend (LinuxSampler)\n"
+			"running in the background. The sampler would continue to work\n"
+			"according to your current sampler session and you could alter the\n"
+			"sampler session at any time by relaunching QSampler.\n\n"
+			"Do you want LinuxSampler to stop?"),
+			QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
+		{
+			bForceServerStop = false;
+		}
+	}
+
 	// And try to stop server.
-	if (m_pServer) {
+	if (m_pServer && bForceServerStop) {
 		appendMessages(tr("Server is stopping..."));
 		if (m_pServer->state() == QProcess::Running) {
 #if defined(WIN32)
@@ -2504,7 +2649,7 @@ void MainForm::processServerExit (void)
 	if (m_pMessages)
 		m_pMessages->flushStdoutBuffer();
 
-	if (m_pServer) {
+	if (m_pServer && bForceServerStop) {
 		if (m_pServer->state() != QProcess::NotRunning) {
 			appendMessages(tr("Server is being forced..."));
 			// Force final server shutdown...
@@ -2588,8 +2733,33 @@ bool MainForm::startClient (void)
 		.arg(::lscp_client_get_timeout(m_pClient)));
 
 	// Subscribe to channel info change notifications...
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_CHANNEL_COUNT) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(CHANNEL_COUNT)");
 	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_CHANNEL_INFO) != LSCP_OK)
-		appendMessagesClient("lscp_client_subscribe");
+		appendMessagesClient("lscp_client_subscribe(CHANNEL_INFO)");
+
+	DeviceStatusForm::onDevicesChanged(); // initialize
+	updateViewMidiDeviceStatusMenu();
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_MIDI_INPUT_DEVICE_COUNT) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(MIDI_INPUT_DEVICE_COUNT)");
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_MIDI_INPUT_DEVICE_INFO) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(MIDI_INPUT_DEVICE_INFO)");
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_AUDIO_OUTPUT_DEVICE_COUNT) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(AUDIO_OUTPUT_DEVICE_COUNT)");
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_AUDIO_OUTPUT_DEVICE_INFO) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(AUDIO_OUTPUT_DEVICE_INFO)");
+
+#if CONFIG_EVENT_CHANNEL_MIDI
+	// Subscribe to channel MIDI data notifications...
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_CHANNEL_MIDI) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(CHANNEL_MIDI)");
+#endif
+
+#if CONFIG_EVENT_DEVICE_MIDI
+	// Subscribe to channel MIDI data notifications...
+	if (::lscp_client_subscribe(m_pClient, LSCP_EVENT_DEVICE_MIDI) != LSCP_OK)
+		appendMessagesClient("lscp_client_subscribe(DEVICE_MIDI)");
+#endif
 
 	// We may stop scheduling around.
 	stopSchedule();
@@ -2616,6 +2786,9 @@ bool MainForm::startClient (void)
 		}
 	}
 
+	// send the current / loaded fine tuning settings to the sampler
+	m_pOptions->sendFineTuningSettings();
+
 	// Make a new session
 	return newSession();
 }
@@ -2643,7 +2816,18 @@ void MainForm::stopClient (void)
 	closeSession(false);
 
 	// Close us as a client...
+#if CONFIG_EVENT_DEVICE_MIDI
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_DEVICE_MIDI);
+#endif
+#if CONFIG_EVENT_CHANNEL_MIDI
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_CHANNEL_MIDI);
+#endif
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_AUDIO_OUTPUT_DEVICE_INFO);
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_AUDIO_OUTPUT_DEVICE_COUNT);
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_MIDI_INPUT_DEVICE_INFO);
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_MIDI_INPUT_DEVICE_COUNT);
 	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_CHANNEL_INFO);
+	::lscp_client_unsubscribe(m_pClient, LSCP_EVENT_CHANNEL_COUNT);
 	::lscp_client_destroy(m_pClient);
 	m_pClient = NULL;
 
