@@ -1,7 +1,7 @@
 // qsamplerMessages.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2008, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -24,8 +24,11 @@
 #include "qsamplerMessages.h"
 
 #include <QSocketNotifier>
+
+#include <QFile>
 #include <QTextEdit>
 #include <QTextCursor>
+#include <QTextStream>
 #include <QTextBlock>
 #include <QScrollBar>
 #include <QDateTime>
@@ -52,32 +55,34 @@ namespace QSampler {
 
 // Constructor.
 Messages::Messages ( QWidget *pParent )
-    : QDockWidget(pParent)
+	: QDockWidget(pParent)
 {
 	// Surely a name is crucial (e.g.for storing geometry settings)
 	QDockWidget::setObjectName("qsamplerMessages");
 
-    // Intialize stdout capture stuff.
-    m_pStdoutNotifier = NULL;
-    m_fdStdout[QSAMPLER_MESSAGES_FDREAD]  = QSAMPLER_MESSAGES_FDNIL;
-    m_fdStdout[QSAMPLER_MESSAGES_FDWRITE] = QSAMPLER_MESSAGES_FDNIL;
+	// Intialize stdout capture stuff.
+	m_pStdoutNotifier = NULL;
+	m_fdStdout[QSAMPLER_MESSAGES_FDREAD]  = QSAMPLER_MESSAGES_FDNIL;
+	m_fdStdout[QSAMPLER_MESSAGES_FDWRITE] = QSAMPLER_MESSAGES_FDNIL;
 
 	// Create local text view widget.
-	m_pTextView = new QTextEdit(this);
-//  QFont font(m_pTextView->font());
+	m_pMessagesTextView = new QTextEdit(this);
+//  QFont font(m_pMessagesTextView->font());
 //  font.setFamily("Fixed");
-//  m_pTextView->setFont(font);
-	m_pTextView->setLineWrapMode(QTextEdit::NoWrap);
-	m_pTextView->setReadOnly(true);
-	m_pTextView->setUndoRedoEnabled(false);
-//	m_pTextView->setTextFormat(Qt::LogText);
+//  m_pMessagesTextView->setFont(font);
+	m_pMessagesTextView->setLineWrapMode(QTextEdit::NoWrap);
+	m_pMessagesTextView->setReadOnly(true);
+	m_pMessagesTextView->setUndoRedoEnabled(false);
+//	m_pMessagesTextView->setTextFormat(Qt::LogText);
 
 	// Initialize default message limit.
 	m_iMessagesLines = 0;
 	setMessagesLimit(QSAMPLER_MESSAGES_MAXLINES);
 
+	m_pMessagesLog = NULL;
+
 	// Prepare the dockable window stuff.
-	QDockWidget::setWidget(m_pTextView);
+	QDockWidget::setWidget(m_pMessagesTextView);
 	QDockWidget::setFeatures(QDockWidget::AllDockWidgetFeatures);
 	QDockWidget::setAllowedAreas(
 		Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
@@ -95,18 +100,21 @@ Messages::Messages ( QWidget *pParent )
 // Destructor.
 Messages::~Messages (void)
 {
-    // No more notifications.
-    if (m_pStdoutNotifier)
-        delete m_pStdoutNotifier;
+	// Turn off and close logging.
+	setLogging(false);
 
-    // No need to delete child widgets, Qt does it all for us.
+	// No more notifications.
+	if (m_pStdoutNotifier)
+		delete m_pStdoutNotifier;
+
+	// No need to delete child widgets, Qt does it all for us.
 }
 
 
-void Messages::showEvent (QShowEvent* event)
+void Messages::showEvent ( QShowEvent *pEvent )
 {
-    QDockWidget::showEvent(event);
-    emit visibilityChanged(isVisible());
+	QDockWidget::showEvent(pEvent);
+	emit visibilityChanged(isVisible());
 }
 
 
@@ -114,12 +122,12 @@ void Messages::showEvent (QShowEvent* event)
 void Messages::stdoutNotify ( int fd )
 {
 #if !defined(WIN32)
-    char achBuffer[1024];
-    int  cchBuffer = ::read(fd, achBuffer, sizeof(achBuffer) - 1);
-    if (cchBuffer > 0) {
-        achBuffer[cchBuffer] = (char) 0;
-        appendStdoutBuffer(achBuffer);
-    }
+	char achBuffer[1024];
+	int  cchBuffer = ::read(fd, achBuffer, sizeof(achBuffer) - 1);
+	if (cchBuffer > 0) {
+		achBuffer[cchBuffer] = (char) 0;
+		appendStdoutBuffer(achBuffer);
+	}
 #endif
 }
 
@@ -144,23 +152,23 @@ void Messages::appendStdoutBuffer ( const QString& s )
 // Stdout flusher -- show up any unfinished line...
 void Messages::flushStdoutBuffer (void)
 {
-    if (!m_sStdoutBuffer.isEmpty()) {
-        appendMessagesText(m_sStdoutBuffer);
-        m_sStdoutBuffer.truncate(0);
-    }
+	if (!m_sStdoutBuffer.isEmpty()) {
+		appendMessagesText(m_sStdoutBuffer);
+		m_sStdoutBuffer.truncate(0);
+	}
 }
 
 
 // Stdout capture accessors.
 bool Messages::isCaptureEnabled (void)
 {
-    return (bool) (m_pStdoutNotifier != NULL);
+	return (bool) (m_pStdoutNotifier != NULL);
 }
 
 void Messages::setCaptureEnabled ( bool bCapture )
 {
-    // Flush current buffer.
-    flushStdoutBuffer();
+	// Flush current buffer.
+	flushStdoutBuffer();
 
 #if !defined(WIN32)
 	// Destroy if already enabled.
@@ -194,47 +202,72 @@ void Messages::setCaptureEnabled ( bool bCapture )
 // Message font accessors.
 QFont Messages::messagesFont (void)
 {
-    return m_pTextView->font();
+	return m_pMessagesTextView->font();
 }
 
-void Messages::setMessagesFont( const QFont& font )
+void Messages::setMessagesFont ( const QFont& font )
 {
-    m_pTextView->setFont(font);
+	m_pMessagesTextView->setFont(font);
 }
 
 
 // Maximum number of message lines accessors.
 int Messages::messagesLimit (void)
 {
-    return m_iMessagesLimit;
+	return m_iMessagesLimit;
 }
 
 void Messages::setMessagesLimit ( int iMessagesLimit )
 {
-    m_iMessagesLimit = iMessagesLimit;
-    m_iMessagesHigh  = iMessagesLimit + (iMessagesLimit / 3);
+	m_iMessagesLimit = iMessagesLimit;
+	m_iMessagesHigh  = iMessagesLimit + (iMessagesLimit / 3);
+}
+
+// Messages logging stuff.
+bool Messages::isLogging (void) const
+{
+	return (m_pMessagesLog != NULL);
+}
+
+void Messages::setLogging ( bool bEnabled, const QString& sFilename )
+{
+	if (m_pMessagesLog) {
+		appendMessages(tr("Logging stopped --- %1 ---")
+			.arg(QDateTime::currentDateTime().toString()));
+		m_pMessagesLog->close();
+		delete m_pMessagesLog;
+		m_pMessagesLog = NULL;
+	}
+
+	if (bEnabled) {
+		m_pMessagesLog = new QFile(sFilename);
+		if (m_pMessagesLog->open(QIODevice::Text | QIODevice::Append)) {
+			appendMessages(tr("Logging started --- %1 ---")
+				.arg(QDateTime::currentDateTime().toString()));
+		} else {
+			delete m_pMessagesLog;
+			m_pMessagesLog = NULL;
+		}
+	}
 }
 
 
-// The main utility methods.
-void Messages::appendMessages ( const QString& s )
+// Messages log output method.
+void Messages::appendMessagesLog ( const QString& s )
 {
-    appendMessagesColor(s, "#999999");
+	if (m_pMessagesLog) {
+		QTextStream(m_pMessagesLog) << s << endl;
+		m_pMessagesLog->flush();
+	}
 }
 
-void Messages::appendMessagesColor ( const QString& s, const QString &c )
+// Messages widget output method.
+void Messages::appendMessagesLine ( const QString& s )
 {
-	appendMessagesText("<font color=\"" + c + "\">"
-		+ QTime::currentTime().toString("hh:mm:ss.zzz")
-		+ ' ' + s + "</font>");
-}
-
-void Messages::appendMessagesText ( const QString& s )
-{
-    // Check for message line limit...
-    if (m_iMessagesLines > m_iMessagesHigh) {
-		m_pTextView->setUpdatesEnabled(false);
-		QTextCursor textCursor(m_pTextView->document()->begin());
+	// Check for message line limit...
+	if (m_iMessagesLines > m_iMessagesHigh) {
+		m_pMessagesTextView->setUpdatesEnabled(false);
+		QTextCursor textCursor(m_pMessagesTextView->document()->begin());
 		while (m_iMessagesLines > m_iMessagesLimit) {
 			// Move cursor extending selection
 			// from start to next line-block...
@@ -244,20 +277,39 @@ void Messages::appendMessagesText ( const QString& s )
 		}
 		// Remove the excessive line-blocks...
 		textCursor.removeSelectedText();
-		m_pTextView->setUpdatesEnabled(true);
-    }
+		m_pMessagesTextView->setUpdatesEnabled(true);
+	}
 
-	// Count always as a new line out there...
-	m_pTextView->append(s);
+	m_pMessagesTextView->append(s);
 	m_iMessagesLines++;
 }
 
 
+// The main utility methods.
+void Messages::appendMessages ( const QString& s )
+{
+	appendMessagesColor(s, "#999999");
+}
+
+void Messages::appendMessagesColor ( const QString& s, const QString &c )
+{
+	QString sText = QTime::currentTime().toString("hh:mm:ss.zzz") + ' ' + s;
+	appendMessagesLine("<font color=\"" + c + "\">" + sText + "</font>");
+	appendMessagesLog(sText);
+}
+
+void Messages::appendMessagesText ( const QString& s )
+{
+	appendMessagesLine(s);
+	appendMessagesLog(s);
+}
+
+
 // History reset.
 void Messages::clear (void)
 {
 	m_iMessagesLines = 0;
-	m_pTextView->clear();
+	m_pMessagesTextView->clear();
 }
 
 } // namespace QSampler
