@@ -1,7 +1,7 @@
 // qsamplerOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 
 #include "qsamplerAbout.h"
 #include "qsamplerOptions.h"
+#include "qsamplerMainForm.h"
 
 #include <QTextStream>
 #include <QComboBox>
@@ -50,12 +51,27 @@ Options::Options (void)
 	m_settings.beginGroup("/Server");
 	sServerHost    = m_settings.value("/ServerHost", "localhost").toString();
 	iServerPort    = m_settings.value("/ServerPort", 8888).toInt();
+	#if defined(__APPLE__)  //  Toshi Nagata 20080105
+	//  TODO: Should this be a configure option?
+	iServerTimeout = m_settings.value("/ServerTimeout", 10000).toInt();
+	#else
 	iServerTimeout = m_settings.value("/ServerTimeout", 1000).toInt();
+	#endif
 	bServerStart   = m_settings.value("/ServerStart", true).toBool();
+	#if defined(__APPLE__)  //  Toshi Nagata 20080113
+	sServerCmdLine = m_settings.value("/ServerCmdLine", "linuxsampler.starter").toString();
+	#else
 	sServerCmdLine = m_settings.value("/ServerCmdLine", "linuxsampler").toString();
+	#endif
 	iStartDelay    = m_settings.value("/StartDelay", 3).toInt();
 	m_settings.endGroup();
 
+	// Load logging options...
+	m_settings.beginGroup("/Logging");
+	bMessagesLog     = m_settings.value("/MessagesLog", false).toBool();
+	sMessagesLogPath = m_settings.value("/MessagesLogPath", "qsampler.log").toString();
+	m_settings.endGroup();
+
 	// Load display options...
 	m_settings.beginGroup("/Display");
 	sDisplayFont     = m_settings.value("/DisplayFont").toString();
@@ -71,6 +87,7 @@ Options::Options (void)
 	bStdoutCapture   = m_settings.value("/StdoutCapture", true).toBool();
 	bCompletePath    = m_settings.value("/CompletePath", true).toBool();
 	iMaxRecentFiles  = m_settings.value("/MaxRecentFiles", 5).toInt();
+	iBaseFontSize    = m_settings.value("/BaseFontSize", 0).toInt();
 // if libgig provides a fast way to retrieve instrument names even for large
 // .gig files, then we enable this feature by default
 #if HAVE_LIBGIG_SETAUTOLOAD
@@ -101,6 +118,12 @@ Options::Options (void)
 	}
 	m_settings.endGroup();
 
+	// Sampler fine tuning settings.
+	m_settings.beginGroup("/Tuning");
+	iMaxVoices  = m_settings.value("/MaxVoices",  -1).toInt();
+	iMaxStreams = m_settings.value("/MaxStreams",  -1).toInt();
+	m_settings.endGroup();
+
 	// Last but not least, get the default directories.
 	m_settings.beginGroup("/Default");
 	sSessionDir    = m_settings.value("/SessionDir").toString();
@@ -138,6 +161,12 @@ Options::~Options (void)
 	m_settings.setValue("/StartDelay", iStartDelay);
 	m_settings.endGroup();
 
+	// Save logging options...
+	m_settings.beginGroup("/Logging");
+	m_settings.setValue("/MessagesLog", bMessagesLog);
+	m_settings.setValue("/MessagesLogPath", sMessagesLogPath);
+	m_settings.endGroup();
+
 	// Save display options.
 	m_settings.beginGroup("/Display");
 	m_settings.setValue("/DisplayFont", sDisplayFont);
@@ -153,6 +182,7 @@ Options::~Options (void)
 	m_settings.setValue("/StdoutCapture", bStdoutCapture);
 	m_settings.setValue("/CompletePath", bCompletePath);
 	m_settings.setValue("/MaxRecentFiles", iMaxRecentFiles);
+	m_settings.setValue("/BaseFontSize", iBaseFontSize);
 	m_settings.setValue("/InstrumentNames", bInstrumentNames);
 	m_settings.endGroup();
 
@@ -174,6 +204,14 @@ Options::~Options (void)
 		m_settings.setValue("/File" + QString::number(++iFile), iter.next());
 	m_settings.endGroup();
 
+	// Sampler fine tuning settings.
+	m_settings.beginGroup("/Tuning");
+	if (iMaxVoices > 0)
+		m_settings.setValue("/MaxVoices", iMaxVoices);
+	if (iMaxStreams >= 0)
+		m_settings.setValue("/MaxStreams", iMaxStreams);
+	m_settings.endGroup();
+
 	// Default directories.
 	m_settings.beginGroup("/Default");
 	m_settings.setValue("/SessionDir", sSessionDir);
@@ -204,7 +242,7 @@ QSettings& Options::settings (void)
 //
 
 // Help about command line options.
-void Options::print_usage ( const char *arg0 )
+void Options::print_usage ( const QString& arg0 )
 {
 	QTextStream out(stderr);
 	out << QObject::tr("Usage: %1 [options] [session-file]\n\n"
@@ -220,30 +258,31 @@ void Options::print_usage ( const char *arg0 )
 
 
 // Parse command line arguments into m_settings.
-bool Options::parse_args ( int argc, char **argv )
+bool Options::parse_args ( const QStringList& args )
 {
 	QTextStream out(stderr);
 	const QString sEol = "\n\n";
 	int iCmdArgs = 0;
+	int argc = args.count();
 
 	for (int i = 1; i < argc; i++) {
 
 		if (iCmdArgs > 0) {
 			sSessionFile += " ";
-			sSessionFile += argv[i];
+			sSessionFile += args.at(i);
 			iCmdArgs++;
 			continue;
 		}
 
-		QString sArg = argv[i];
+		QString sArg = args.at(i);
 		QString sVal = QString::null;
 		int iEqual = sArg.indexOf("=");
 		if (iEqual >= 0) {
 			sVal = sArg.right(sArg.length() - iEqual - 1);
 			sArg = sArg.left(iEqual);
 		}
-		else if (i < argc)
-			sVal = argv[i + 1];
+		else if (i < argc - 1)
+			sVal = args.at(i + 1);
 
 		if (sArg == "-s" || sArg == "--start") {
 			bServerStart = true;
@@ -267,7 +306,7 @@ bool Options::parse_args ( int argc, char **argv )
 				i++;
 		}
 		else if (sArg == "-?" || sArg == "--help") {
-			print_usage(argv[0]);
+			print_usage(args.at(0));
 			return false;
 		}
 		else if (sArg == "-v" || sArg == "--version") {
@@ -402,6 +441,99 @@ void Options::saveComboBoxHistory ( QComboBox *pComboBox, int iLimit )
 	m_settings.endGroup();
 }
 
+int Options::getMaxVoices() {
+#ifndef CONFIG_MAX_VOICES
+	return -1;
+#else
+	if (iMaxVoices > 0) return iMaxVoices;
+	return getEffectiveMaxVoices();
+#endif // CONFIG_MAX_VOICES
+}
+
+int Options::getEffectiveMaxVoices() {
+#ifndef CONFIG_MAX_VOICES
+	return -1;
+#else
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !pMainForm->client())
+		return -1;
+
+	return ::lscp_get_voices(pMainForm->client());
+#endif // CONFIG_MAX_VOICES
+}
+
+void Options::setMaxVoices(int iMaxVoices) {
+#ifdef CONFIG_MAX_VOICES
+	if (iMaxVoices < 1) return;
+
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !pMainForm->client())
+		return;
+
+	lscp_status_t result =
+		::lscp_set_voices(pMainForm->client(), iMaxVoices);
+
+	if (result != LSCP_OK) {
+		pMainForm->appendMessagesClient("lscp_set_voices");
+		return;
+	}
+
+	this->iMaxVoices = iMaxVoices;
+#endif // CONFIG_MAX_VOICES
+}
+
+int Options::getMaxStreams() {
+#ifndef CONFIG_MAX_VOICES
+	return -1;
+#else
+	if (iMaxStreams > 0) return iMaxStreams;
+	return getEffectiveMaxStreams();
+#endif // CONFIG_MAX_VOICES
+}
+
+int Options::getEffectiveMaxStreams() {
+#ifndef CONFIG_MAX_VOICES
+	return -1;
+#else
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !pMainForm->client())
+		return -1;
+
+	return ::lscp_get_streams(pMainForm->client());
+#endif // CONFIG_MAX_VOICES
+}
+
+void Options::setMaxStreams(int iMaxStreams) {
+#ifdef CONFIG_MAX_VOICES
+	if (iMaxStreams < 0) return;
+
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !pMainForm->client())
+		return;
+
+	lscp_status_t result =
+		::lscp_set_streams(pMainForm->client(), iMaxStreams);
+
+	if (result != LSCP_OK) {
+		pMainForm->appendMessagesClient("lscp_set_streams");
+		return;
+	}
+
+	this->iMaxStreams = iMaxStreams;
+#endif // CONFIG_MAX_VOICES
+}
+
+void Options::sendFineTuningSettings() {
+	setMaxVoices(iMaxVoices);
+	setMaxStreams(iMaxStreams);
+
+	MainForm *pMainForm = MainForm::getInstance();
+	if (!pMainForm || !pMainForm->client())
+		return;
+
+	pMainForm->appendMessages(QObject::tr("Sent fine tuning settings."));
+}
+
 } // namespace QSampler
 
 
