@@ -1,7 +1,7 @@
 // qsamplerOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -49,9 +49,9 @@ public:
 	QSettings& settings();
 
 	// Command line arguments parser.
-	bool parse_args(int argc, char **argv);
+	bool parse_args(const QStringList& args);
 	// Command line usage helper.
-	void print_usage(const char *arg0);
+	void print_usage(const QString& arg0);
 
 	// Startup supplied session file.
 	QString sSessionFile;
@@ -64,6 +64,10 @@ public:
 	QString sServerCmdLine;
 	int     iStartDelay;
 
+	// Logging options...
+	bool    bMessagesLog;
+	QString sMessagesLogPath;
+
 	// Display options...
 	QString sDisplayFont;
 	bool    bDisplayEffect;
@@ -78,6 +82,7 @@ public:
 	bool    bStdoutCapture;
 	bool    bCompletePath;
 	bool    bInstrumentNames;
+	int     iBaseFontSize;
 
 	// View options...
 	bool    bMenubar;
@@ -109,10 +114,24 @@ public:
 	void loadComboBoxHistory(QComboBox *pComboBox, int iLimit = 8);
 	void saveComboBoxHistory(QComboBox *pComboBox, int iLimit = 8);
 
+	int  getMaxVoices();
+	int  getEffectiveMaxVoices();
+	void setMaxVoices(int iMaxVoices);
+
+	int  getMaxStreams();
+	int  getEffectiveMaxStreams();
+	void setMaxStreams(int iMaxStreams);
+
+	void sendFineTuningSettings();
+
 private:
 
 	// Settings member variables.
 	QSettings m_settings;
+
+	// Tuning
+	int iMaxVoices;
+	int iMaxStreams;
 };
 
 } // namespace QSampler
