@@ -1,7 +1,7 @@
 // qsamplerOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -27,6 +27,7 @@
 
 #include <QMessageBox>
 #include <QFontDialog>
+#include <QFileDialog>
 
 
 namespace QSampler {
@@ -72,6 +73,15 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	QObject::connect(m_ui.StartDelaySpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.MessagesLogCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.MessagesLogPathComboBox,
+		SIGNAL(editTextChanged(const QString&)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.MessagesLogPathToolButton,
+		SIGNAL(clicked()),
+		SLOT(browseMessagesLogPath()));
 	QObject::connect(m_ui.DisplayFontPushButton,
 		SIGNAL(clicked()),
 		SLOT(chooseDisplayFont()));
@@ -114,6 +124,15 @@ OptionsForm::OptionsForm ( QWidget* pParent )
 	QObject::connect(m_ui.InstrumentNamesCheckBox,
 		SIGNAL(stateChanged(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.BaseFontSizeComboBox,
+		SIGNAL(editTextChanged(const QString&)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.MaxVoicesSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(maxVoicesChanged(int)));
+	QObject::connect(m_ui.MaxStreamsSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(maxStreamsChanged(int)));
 	QObject::connect(m_ui.OkPushButton,
 		SIGNAL(clicked()),
 		SLOT(accept()));
@@ -141,6 +160,7 @@ void OptionsForm::setup ( Options *pOptions )
 	m_pOptions->loadComboBoxHistory(m_ui.ServerHostComboBox);
 	m_pOptions->loadComboBoxHistory(m_ui.ServerPortComboBox);
 	m_pOptions->loadComboBoxHistory(m_ui.ServerCmdLineComboBox);
+	m_pOptions->loadComboBoxHistory(m_ui.MessagesLogPathComboBox);
 
 	// Load Server settings...
 	m_ui.ServerHostComboBox->setEditText(m_pOptions->sServerHost);
@@ -150,6 +170,10 @@ void OptionsForm::setup ( Options *pOptions )
 	m_ui.ServerCmdLineComboBox->setEditText(m_pOptions->sServerCmdLine);
 	m_ui.StartDelaySpinBox->setValue(m_pOptions->iStartDelay);
 
+	// Logging options...
+	m_ui.MessagesLogCheckBox->setChecked(m_pOptions->bMessagesLog);
+	m_ui.MessagesLogPathComboBox->setEditText(m_pOptions->sMessagesLogPath);
+
 	// Load Display options...
 	QFont font;
 	QPalette pal;
@@ -174,9 +198,9 @@ void OptionsForm::setup ( Options *pOptions )
 	// Messages font.
 	if (m_pOptions->sMessagesFont.isEmpty()
 		|| !font.fromString(m_pOptions->sMessagesFont))
-		font = QFont("Fixed", 8);
+		font = QFont("Monospace", 8);
 	pal = m_ui.MessagesFontTextLabel->palette();
-	pal.setColor(QPalette::Background, Qt::white);
+	pal.setColor(QPalette::Background, pal.base().color());
 	m_ui.MessagesFontTextLabel->setPalette(pal);
 	m_ui.MessagesFontTextLabel->setFont(font);
 	m_ui.MessagesFontTextLabel->setText(font.family()
@@ -193,11 +217,58 @@ void OptionsForm::setup ( Options *pOptions )
 	m_ui.CompletePathCheckBox->setChecked(m_pOptions->bCompletePath);
 	m_ui.InstrumentNamesCheckBox->setChecked(m_pOptions->bInstrumentNames);
 	m_ui.MaxRecentFilesSpinBox->setValue(m_pOptions->iMaxRecentFiles);
+	if (m_pOptions->iBaseFontSize > 0)
+		m_ui.BaseFontSizeComboBox->setEditText(QString::number(m_pOptions->iBaseFontSize));
+	else
+		m_ui.BaseFontSizeComboBox->setCurrentIndex(0);
 
 #ifndef CONFIG_LIBGIG
 	m_ui.InstrumentNamesCheckBox->setEnabled(false);
 #endif
 
+	bMaxVoicesModified = bMaxStreamsModified = false;
+#ifdef CONFIG_MAX_VOICES
+	const bool bMaxVoicesSupported =
+		m_pOptions->getEffectiveMaxVoices() >= 0;
+	const bool bMaxStreamsSupported =
+		m_pOptions->getEffectiveMaxStreams() >= 0;
+
+	m_ui.MaxVoicesSpinBox->setEnabled(bMaxVoicesSupported);
+	m_ui.MaxVoicesSpinBox->setValue(m_pOptions->getMaxVoices());
+	if (!bMaxVoicesSupported)
+		m_ui.MaxVoicesSpinBox->setToolTip(
+			tr("This parameter is not supported by the current sampler "
+			   "version in use.")
+		);
+	else
+		m_ui.MaxVoicesSpinBox->setToolTip(
+			tr("The max. amount of voices the sampler shall process "
+			   "simultaniously.")
+		);
+
+	m_ui.MaxStreamsSpinBox->setEnabled(bMaxStreamsSupported);
+	m_ui.MaxStreamsSpinBox->setValue(m_pOptions->getMaxStreams());
+	if (!bMaxStreamsSupported)
+		m_ui.MaxStreamsSpinBox->setToolTip(
+			tr("This parameter is not supported by the current sampler "
+			   "version in use.")
+		);
+	else
+		m_ui.MaxStreamsSpinBox->setToolTip(
+			tr("The max. amount of disk streams the sampler shall process "
+			   "simultaniously.")
+		);
+#else
+	m_ui.MaxVoicesSpinBox->setEnabled(false);
+	m_ui.MaxStreamsSpinBox->setEnabled(false);
+	m_ui.MaxVoicesSpinBox->setToolTip(
+		tr("QSampler was built without support for this parameter.")
+	);
+	m_ui.MaxStreamsSpinBox->setToolTip(
+		tr("QSampler was built without support for this parameter.")
+	);
+#endif // CONFIG_MAX_VOICES
+
 	// Done.
 	m_iDirtySetup--;
 	stabilizeForm();
@@ -215,7 +286,10 @@ void OptionsForm::accept (void)
 		m_pOptions->iServerTimeout = m_ui.ServerTimeoutSpinBox->value();
 		m_pOptions->bServerStart   = m_ui.ServerStartCheckBox->isChecked();
 		m_pOptions->sServerCmdLine = m_ui.ServerCmdLineComboBox->currentText().trimmed();
-		m_pOptions->iStartDelay      = m_ui.StartDelaySpinBox->value();
+		m_pOptions->iStartDelay    = m_ui.StartDelaySpinBox->value();
+		// Logging options...
+		m_pOptions->bMessagesLog   = m_ui.MessagesLogCheckBox->isChecked();
+		m_pOptions->sMessagesLogPath = m_ui.MessagesLogPathComboBox->currentText();
 		// Channels options...
 		m_pOptions->sDisplayFont   = m_ui.DisplayFontTextLabel->font().toString();
 		m_pOptions->bDisplayEffect = m_ui.DisplayEffectCheckBox->isChecked();
@@ -233,14 +307,23 @@ void OptionsForm::accept (void)
 		m_pOptions->bCompletePath  = m_ui.CompletePathCheckBox->isChecked();
 		m_pOptions->bInstrumentNames = m_ui.InstrumentNamesCheckBox->isChecked();
 		m_pOptions->iMaxRecentFiles  = m_ui.MaxRecentFilesSpinBox->value();
+		m_pOptions->iBaseFontSize  = m_ui.BaseFontSizeComboBox->currentText().toInt();
 		// Reset dirty flag.
 		m_iDirtyCount = 0;
 	}
 
+	// if the user modified the limits, apply them to the sampler
+	// (and store it later in qsampler's configuration)
+	if (bMaxVoicesModified && m_ui.MaxVoicesSpinBox->isEnabled())
+		m_pOptions->setMaxVoices(m_ui.MaxVoicesSpinBox->value());
+	if (bMaxStreamsModified && m_ui.MaxStreamsSpinBox->isEnabled())
+		m_pOptions->setMaxStreams(m_ui.MaxStreamsSpinBox->value());
+
 	// Save combobox history...
 	m_pOptions->saveComboBoxHistory(m_ui.ServerHostComboBox);
 	m_pOptions->saveComboBoxHistory(m_ui.ServerPortComboBox);
 	m_pOptions->saveComboBoxHistory(m_ui.ServerCmdLineComboBox);
+	m_pOptions->saveComboBoxHistory(m_ui.MessagesLogPathComboBox);
 
 	// Just go with dialog acceptance.
 	QDialog::accept();
@@ -258,11 +341,13 @@ void OptionsForm::reject (void)
 			QSAMPLER_TITLE ": " + tr("Warning"),
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
-			tr("Apply"), tr("Discard"), tr("Cancel"))) {
-		case 0:     // Apply...
+			QMessageBox::Apply |
+			QMessageBox::Discard |
+			QMessageBox::Cancel)) {
+		case QMessageBox::Apply:
 			accept();
 			return;
-		case 1:     // Discard
+		case QMessageBox::Discard:
 			break;
 		default:    // Cancel.
 			bReject = false;
@@ -288,18 +373,46 @@ void OptionsForm::optionsChanged (void)
 // Stabilize current form state.
 void OptionsForm::stabilizeForm (void)
 {
+	bool bValid = (m_iDirtyCount > 0);
+
 	bool bEnabled = m_ui.ServerStartCheckBox->isChecked();
 	m_ui.ServerCmdLineTextLabel->setEnabled(bEnabled);
 	m_ui.ServerCmdLineComboBox->setEnabled(bEnabled);
 	m_ui.StartDelayTextLabel->setEnabled(bEnabled);
 	m_ui.StartDelaySpinBox->setEnabled(bEnabled);
 
+	bEnabled = m_ui.MessagesLogCheckBox->isChecked();
+	m_ui.MessagesLogPathComboBox->setEnabled(bEnabled);
+	m_ui.MessagesLogPathToolButton->setEnabled(bEnabled);
+	if (bEnabled && bValid) {
+		const QString& sPath = m_ui.MessagesLogPathComboBox->currentText();
+		bValid = !sPath.isEmpty();
+	}
+
 	m_ui.AutoRefreshTimeSpinBox->setEnabled(
 		m_ui.AutoRefreshCheckBox->isChecked());
 	m_ui.MessagesLimitLinesSpinBox->setEnabled(
 		m_ui.MessagesLimitCheckBox->isChecked());
 
-	m_ui.OkPushButton->setEnabled(m_iDirtyCount > 0);
+	m_ui.OkPushButton->setEnabled(bValid);
+}
+
+
+// Messages log path browse slot.
+void OptionsForm::browseMessagesLogPath (void)
+{
+	QString sFileName = QFileDialog::getSaveFileName(
+		this,											// Parent.
+		tr("Messages Log"),				                // Caption.
+		m_ui.MessagesLogPathComboBox->currentText(),	// Start here.
+		tr("Log files") + " (*.log)"	                // Filter (log files)
+	);
+
+	if (!sFileName.isEmpty()) {
+		m_ui.MessagesLogPathComboBox->setEditText(sFileName);
+		m_ui.MessagesLogPathComboBox->setFocus();
+		optionsChanged();
+	}
 }
 
 
@@ -349,7 +462,18 @@ void OptionsForm::toggleDisplayEffect ( bool bOn )
 	optionsChanged();
 }
 
+void OptionsForm::maxVoicesChanged(int /*iMaxVoices*/)
+{
+	bMaxVoicesModified = true;
+	optionsChanged();
+}
+
+void OptionsForm::maxStreamsChanged(int /*iMaxStreams*/)
+{
+	bMaxStreamsModified = true;
+	optionsChanged();
+}
+
 } // namespace QSampler
 
-
 // end of qsamplerOptionsForm.cpp
