@@ -1,6 +1,6 @@
 
 #define PACKAGE_NAME    "Qsampler"
-#define PACKAGE_VERSION "0.2.1"
+#define PACKAGE_VERSION "0.2.2"
 
 #define CONFIG_PREFIX   "."
 #define CONFIG_DEBUG    1
@@ -16,8 +16,14 @@
 #define CONFIG_AUDIO_ROUTING    1
 #define CONFIG_FXSEND           1
 #define CONFIG_FXSEND_LEVEL     1
+#define CONFIG_FXSEND_RENAME    1
 #define CONFIG_VOLUME           1
 
+#define CONFIG_EVENT_CHANNEL_MIDI 1
+#define CONFIG_EVENT_DEVICE_MIDI  1
+
+#define CONFIG_MAX_VOICES       1
+
 #undef  HAVE_SIGNAL_H
 
 
